<!--===============================================-->
<!--  TMNDoublePQ.xsl                              -->
<!--  The interpreter of the Turing machine        -->
<!--===============================================-->
<!-- Example of the program for the Turing machine -->
<!-- Multiplication on 2 in a unary notation       -->
<!-- Processing of borders of the tape             -->
<!--===============================================-->


<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xt="http://www.jclark.com/xt"
		        version="1.0">

<!-- TMDoublePQ.xsl
*******************************************************************************
*  Alexandr Korlyukov.                                                        *
*     An interpreter of Turing machine interprets a concrete Turing machine.  *
*     Initially, the concrete machine is put inside a Program                 *
*     ( id est the concrete machine is just a part of this XSLT-program).     *
*                                                                             *
*     An Instruction has the form                                             *
*     (CurrentState   CurrentSymbol   NextSymbol   NextState   Move)          *
*     This instruction is executable if the machine is in the configuration   *
*     (CurrentState   CurrentSymbol), that is , its internal state is         *
*     CurrentState and the symbol scanned under the head is CurrentSymbol.    *
*     Its execution consits in overwriting the square under the head by symbol*
*     NextSymbol, in moving the head on the tape one square from the present  *
*     square in the direction indicated by Move and in changing the internal  *
*     state into NextState.                                                   *
*     A program consists of a finite number instructions.There is at least one*
*     instruction with the start state "start". There can be istructions with *
*     the finish state "stop".                                                *
*     The program starts with an instruction with the start state.            *
*     The execution of the program consists in repeatedly executing any one   *
*     of the executable instructions. The execution halts when no instruction *
*     is executable.                                                          *
*     Initially, the tape contains infinitely many blank symbols "B' before   *
*     and after of finitely many nonblanks symbols in a neighbourhood         *
*     of the head.                                                            *
*******************************************************************************
-->

<!-- The concrete Turing machine: replace P with double Q on a tape.
     The "P'-symbols are replaced on the right hand side to the head.
-->

<!-- ****************************************************** -->
<!--  The interpreter itself.                               -->
<!-- ****************************************************** -->

<xsl:template match="Go">
	<xsl:call-template name="Go2">
	<xsl:with-param name="TMS">
	<TMState>
        <Instruction CurrState="start"    CurrSymb="B" NextSymb="B" NextState="stop"     Movement="right">
        <Instruction CurrState="start"    CurrSymb="Q" NextSymb="Q" NextState="start"    Movement="right">
        <Instruction CurrState="start"    CurrSymb="P" NextSymb="Q" NextState="moveleft" Movement="left" >
        <Instruction CurrState="moveleft" CurrSymb="Q" NextSymb="Q" NextState="moveleft" Movement="left" >
        <Instruction CurrState="moveleft" CurrSymb="B" NextSymb="Q" NextState="start"    Movement="right">
        </Instruction>
        </Instruction>
        </Instruction>
        </Instruction>
        </Instruction>
        <!-- The end of the concrete Turing machine. -->
		<State>start</State>
		<LeftTape>
		<Nod><Cell>B</Cell><Nod><Cell>B</Cell>
		<Nod><N/></Nod>
		</Nod></Nod>
		</LeftTape>
		<xsl:copy-of select="Symbol"/>
		<xsl:copy-of select="RightTape"/>
	</TMState>
	</xsl:with-param>
	</xsl:call-template>

</xsl:template>

<!--===============================================-->
<!--        Head function                          -->
<!--===============================================-->

<xsl:template name="Go2"> 
<xsl:param name="TMS"/>
<xsl:for-each select="xt:node-set($TMS)/TMState">

<xsl:choose>

<xsl:when test="State='stop'">
    <tm>
	<xsl:copy-of select="LeftTape"/>
	<xsl:copy-of select="Symbol"/>
	<xsl:copy-of select="RightTape"/>
	</tm>
</xsl:when>

<xsl:otherwise>
	<xsl:call-template name="Go2">
	<xsl:with-param name="TMS">
	<TMState>
	<xsl:copy-of select="Instruction"/>
		<xsl:call-template name="Step">
		<xsl:with-param name="TMS">
		<TMState>
			<xsl:copy-of select="Instruction"/>
			<xsl:copy-of select="State"/>
			<xsl:copy-of select="LeftTape"/>
			<xsl:copy-of select="Symbol"/>
			<xsl:copy-of select="RightTape"/>
		</TMState>
		</xsl:with-param>
		</xsl:call-template>
	</TMState>
	</xsl:with-param>
	</xsl:call-template>
</xsl:otherwise>

</xsl:choose>
</xsl:for-each>
</xsl:template>

<!--===============================================-->
<!--    One step of the Turing machine             -->
<!--===============================================-->

<xsl:template name="Step">
  <xsl:param name="TMS"/>
<xsl:for-each select="xt:node-set($TMS)/TMState">

<xsl:choose>

<xsl:when test="Instruction/@CurrState=State">
	<xsl:choose>
	<xsl:when test="Instruction/@CurrSymb=Symbol">
		<xsl:choose>
		<xsl:when test="Instruction/@Movement='right'">
			<State>
				<xsl:value-of select="Instruction/@NextState"/>
			</State>
			<LeftTape>
				<Nod>
				<Cell>
				<xsl:value-of select="Instruction/@NextSymb"/>
				</Cell>
				<xsl:copy-of select="LeftTape/Nod"/>
				</Nod>
			</LeftTape>
			<Symbol>
				<xsl:value-of select="RightTape/Nod/Cell"/>
			</Symbol>
			<RightTape>
				<Nod>
				<xsl:for-each select="RightTape/Nod/Nod">
				<xsl:apply-templates/>
				</xsl:for-each>
				</Nod>
			</RightTape>
		</xsl:when>

		<xsl:when test="Instruction/@Movement='left'">
			<State>
				<xsl:value-of select="Instruction/@NextState"/>
			</State>
			<LeftTape>
				<Nod>
				<xsl:for-each select="LeftTape/Nod/Nod">
				<xsl:apply-templates/>
				</xsl:for-each>
				</Nod>
			</LeftTape>
			<Symbol>
				<xsl:value-of select="LeftTape/Nod/Cell"/>
			</Symbol>
			<RightTape>
				<Nod>
				<Cell>
				<xsl:value-of select="Instruction/@NextSymb"/>
				</Cell>
				<xsl:copy-of select="RightTape/Nod"/>
				</Nod>
			</RightTape>
		</xsl:when>
		</xsl:choose>
	</xsl:when>

	<xsl:otherwise>
		<xsl:call-template name="Step">
		<xsl:with-param name="TMS">
		<TMState>
			<xsl:copy-of select="Instruction/Instruction"/>
			<xsl:copy-of select="State"/>
			<xsl:copy-of select="LeftTape"/>
			<xsl:copy-of select="Symbol"/>
			<xsl:copy-of select="RightTape"/>
		</TMState>
		</xsl:with-param>
		</xsl:call-template>
	</xsl:otherwise>
	</xsl:choose>
</xsl:when>

<xsl:otherwise>
	<xsl:call-template name="Step">
	<xsl:with-param name="TMS">
	<TMState>
		<xsl:copy-of select="Instruction/Instruction"/>
		<xsl:copy-of select="State"/>
		<xsl:copy-of select="LeftTape"/>
		<xsl:copy-of select="Symbol"/>
		<xsl:copy-of select="RightTape"/>
	</TMState>
	</xsl:with-param>
	</xsl:call-template>
</xsl:otherwise>

</xsl:choose>
</xsl:for-each>
</xsl:template>

<!--===============================================-->
<!--      Processing of the end of a tape          -->
<!--===============================================-->

<xsl:template match="N">
    <Cell>B</Cell>
    <Nod><Cell>B</Cell>
    <Nod><N/>
    </Nod></Nod>
</xsl:template>

<xsl:template match="Nod">
     <xsl:copy-of select="."/>
</xsl:template>

<xsl:template match="Cell">
    <xsl:copy-of select="."/>
</xsl:template>

<!-- ****************************************************** -->
<!--                 The end.                               -->
<!-- ****************************************************** -->

</xsl:stylesheet>